#ifndef GLWIDGET_H
#define GLWIDGET_H

/**
 * @file
 * Klassendeklaration fuer das OpenGL-Widget
 *
 * @author
 * Christian Harendt (minf2964)
 * Philip Lehmann-Boehm (minf8180)
 */

#include <QGLWidget>

#include "Logic.h"
#include "Light.h"
#include "DisplayList.h"
#include "glm.h"

class GLWidget : public QGLWidget {
	Q_OBJECT

	public:

		/**
		 * Konstruktor
		 * @param logic Die zu dem OpenGL-Widget zugehoerige Logikinstanz
		 */
		GLWidget(Logic * logic);

		/**
		 * Destruktor
		 */
		~GLWidget();

		/**
		 * Liefert dem Eigentuemer des Widgets die minimal benoetigte Groesse des Widgets.
		 * @return Die minimal benoetigte Groesse.
		 */
		QSize minimumSizeHint() const;

		/**
		 * Liefert dem Eigentuemer die regulaer benoetigte Groesse des Widgets.
		 * @return Die regulaer benoetigte Groesse
		 */
		QSize sizeHint() const;


	protected:

		/**
		 * Initialisiert alle mit OpenGL zusammenhaengenden Sachen.
		 */
		void initializeGL();

		/**
		 * Setzt die Projektionsmatrix von OpenGL neu, wenn sich die Fenstergroesse aendert.
		 * @param width Die neue Fensterbreite
		 * @param height Die neue Fensterhoehe
		 */
		void resizeGL(int width, int height);

	private:
		// Der maximale Winkel, in dem sich die Ebene neigt
		static const float maxAngle = 60.0f;

		// Die Logik-Instanz
		Logic *logic;

		// Die Lichtquelle
		PositionedLight * light;

		// Displaylisten fuer Boden, Person und Konus
		DisplayList *radcircle, *radgreen, *person;

        // Importierte OBJ-Models
		GLMmodel *radar, *radartisch, *radardisplay;

		// Die Texturen-IDs
		GLuint textures[6];

		/**
		 * Zeichnet die OpenGL-Szene.
		 */
		void paintGL();
};

#endif
