/**
 * @file
 * Klassenimplementierung fuer die Einheitsfiguren
 *
 * @author
 * Philip Lehmann-Boehm (minf8180)
 */

#include "UnitForms.h"

#include <GL/glu.h>

/**
 * Zeichnet ein Einheitsquadrat.
 */
void UnitForms::drawUnitQuad (void) {
	glBegin(GL_QUADS);
		glNormal3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f (0.5f, 0.5f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f (-0.5f, 0.5f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f (-0.5f, -0.5f, 0.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f (0.5f, -0.5f, 0.0f);
	glEnd();
}

/**
 * Zeichnet eine Einheitsscheibe.
 * @param details Der Detailierungsgrad der Scheibe
 */
void UnitForms::drawUnitDisk(int details) {
	GLUquadricObj * disk;
	disk = gluNewQuadric();
	gluQuadricNormals(disk, GLU_SMOOTH);
	gluQuadricTexture(disk, GL_TRUE);
	gluDisk(disk, 0.0, 0.5, details, 1);
	gluDeleteQuadric(disk);
}

/**
 * Zeichnet einen Einheitswuerfel
 */
void UnitForms::drawUnitCube (void) {
	/* Frontflaeche */
	glPushMatrix ();
		glTranslatef (0.0f, 0.0f, 0.5f);
		drawUnitQuad ();
	glPopMatrix ();

	/* rechte Seitenflaeche */
	glPushMatrix ();
		glRotatef (90.0f, 0.0f, 1.0f, 0.0f);
		glTranslatef (0.0f, 0.0f, 0.5f);
		drawUnitQuad ();
	glPopMatrix ();

	/* Rueckseitenflaeche */
	glPushMatrix ();
		glRotatef (180.0f, 0.0f, 1.0f, 0.0f);
		glTranslatef (0.0f, 0.0f, 0.5f);
		drawUnitQuad ();
	glPopMatrix ();

	/* linke Seitenflaeche */
	glPushMatrix ();
		glRotatef (270.0f, 0.0f, 1.0f, 0.0f);
		glTranslatef (0.0f, 0.0f, 0.5f);
		drawUnitQuad ();
	glPopMatrix ();

	/* Deckelflaeche */
	glPushMatrix ();
		glRotatef (-90.0f, 1.0f, 0.0f, 0.0f);
		glTranslatef (0.0f, 0.0f, 0.5f);
		drawUnitQuad ();
	glPopMatrix ();

	/* Bodenflaeche */
	glPushMatrix ();
		glRotatef (90.0f, 1.0f, 0.0f, 0.0f);
		glTranslatef (0.0f, 0.0f, 0.5f);
		drawUnitQuad ();
	glPopMatrix ();
}

/**
 * Zeichnet eine Einheitskugel
 * @param details Der Detailierungsgrad der Kugel
 */
void UnitForms::drawUnitSphere(GLint details) {
	GLUquadricObj * sphere;
	sphere = gluNewQuadric();
	gluQuadricNormals(sphere, GLU_SMOOTH);
	gluQuadricTexture(sphere, GL_TRUE);
	gluSphere(sphere, 0.5f, details, details);
	gluDeleteQuadric(sphere);
}

/**
 * Zeichnet eines Einheitskonus
 * @param details Der Detailierungsgrad des Konusses
 */
void UnitForms::drawUnitCone(GLint details) {
	GLUquadricObj * cone;
	cone = gluNewQuadric();
	gluQuadricNormals(cone, GLU_SMOOTH);
	gluQuadricTexture(cone, GL_TRUE);
	glPushMatrix();
		glTranslatef(0.0f, 0.0f, -0.5f);
		gluCylinder(cone, 0.5, 0.0, 1.0, details, details);
		glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
		drawUnitDisk(details);
	glPopMatrix();
	gluDeleteQuadric(cone);
}
