/**
 * @file
 * Klassenimplementierung des Clientfensters
 *
 * @author
 * Philip Lehmann-Boehm (minf8180)
 */

#include <QtGui>
#include <Qt>

#include "GLwidget.h"
#include "Window.h"

#include "Settings.h"

char * Window::settingsXML = "clientSettings.xml";

/**
 * Konstruktor
 */
Window::Window() {

	// Krams initialisieren.
	int timeToLive = 0;
	QString host;
	int port;
	timer = new QTimer(this);

	// Einstellungen laden.
	try {
		Settings settings(settingsXML);
		timeToLive = settings.get("timeToLive", 0);
		port = settings.get("port", Window::defPort);
		host = settings.get("host", "localhost");
	} catch (...) {
		QMessageBox::critical(this, tr("Client error"),
									tr("Unable to load the file clientSttings.xml!"));
		close();
		buildUpFlag = false;
		return;
	}

	// Mit dem Server verbinden
	if (!logic.connectToHost(host, port)) {
		QMessageBox::critical(this, tr("Client error"), tr("Could not connect to the server!"));
		close();
		buildUpFlag = false;
		return;
	}

	// Wenn eingestellt wurde, dass die Anwendung nur eine bestimmte Zeit laufen soll, einen
	// Timer zum Beenden starten.
	if (timeToLive > 0) {
		QTimer * killTimer = new QTimer(this);
		connect(killTimer, SIGNAL(timeout()), this, SLOT(close()));
		killTimer->setSingleShot(true);
		killTimer->start(timeToLive * 1000);
	}

	// OpenGL-Widget initialisieren und Zeitmessung sowie Timer starten.
	glWidget = new GLWidget(&logic);
	QHBoxLayout *mainLayout = new QHBoxLayout;
	mainLayout->addWidget(glWidget);
	mainLayout->setMargin(0);
	setLayout(mainLayout);
	setCursor(Qt::BlankCursor);

	time = new QTime();
	time->start();

	connect(timer, SIGNAL(timeout()), this, SLOT(timerFunc()));
	timer->start(20);

	// Es hat alles geklappt... Yeha!
	buildUpFlag = true;

}

/**
 * Destruktor
 */
Window::~Window() {
	delete timer;
}

/**
 * Timerevent
 */
void Window::timerFunc() {
	// Den naechsten Schritt berechnen lassen
	double elapsed = (double)time->elapsed() / 1000.0;

    if (!logic.nextStep(elapsed)) {
		timer->stop();
		QMessageBox::critical(this, tr("Client error"), tr("Server-connection lost!"));
		close();
		return;
	}
	// Zeit neustarten und OpenGL zeichnen lassen.
	time->restart();
	logic.update(elapsed);
	glWidget->updateGL();
}

/**
 * Keyboardevent
 * @param e Die Eventdaten
 */
void Window::keyPressEvent (QKeyEvent * e) {
	switch (e->key()) {
		// Bei Q, q ode Esc beenden.
		case Qt::Key_Escape:
		case Qt::Key_Q:
			close();
		break;
	}
}

/**
 * Liefert, ob der Konstruktor komplett durchgelaufen ist.
 * @return Ob der Konstruktor komplett durchgelaufen ist
 */
bool Window::buildUp() {
	return buildUpFlag;
}
