/*--------------------*/

/* some macros and static variables
for making version and
conditional compilation flags
visible in assembler and object files
ignore in first reading
*/

//static char version[] = "$Id: List.c,v 1.11 2006/12/23 13:32:06 uwe Exp $";


//static char options[] = optionString;

/*--------------------*/

#include "List.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <float.h>
#include <math.h>

/*--------------------*/

/* simple ops as functions */

List
mkEmptyList(void)
{
    return (List) 0;
}

/*--------------------*/

int
isEmptyList(List l)
{
    return l == (List) 0;
}

/*--------------------*/

Element
head(List l)
{
    assert(! isEmptyList(l));
    return l->info;
}

/*--------------------*/

List
tail(List l)
{
    assert(! isEmptyList(l));
    return l->next;
}


/*--------------------*/

List
removeHead(List l)
{
    assert(! isEmptyList(l));
    {
        List res = l->next;
        free(l);
        return res;
    }
}

List
cons(Element e, List l)
{
    /* the only call of malloc */
    List res = (List)malloc(sizeof(List));

    if (! res)
    {
        perror("cons: malloc failed");
        exit(1);
    }
    res->info = e;
    res->next = l;

    return res;
}

/*--------------------*/

unsigned int
length(List l)
{
    return isEmptyList(l)
           ? 0
           : 1 + length(tail(l));
}



/*--------------------*/

Element
at(List l, unsigned int i)
{
    return i == 0
           ? head(l)
           : at(tail(l), i - 1);
}
/*--------------------*/


/*--------------------*/


bool eqElement(Element x, Element y)
{
    return ((fabs(x.x - y.x) <= DBL_EPSILON) &&
        (fabs(x.y - y.y) <= DBL_EPSILON));
}

bool geElement(Element x, Element y)
{
    return ((x.x >= y.x) &&
        (x.y >= y.y));
}

int
isInList(Element e, List l)
{
    if (isEmptyList(l))
        return 0;

    if (eqElement(e, l->info))
        return 1;

    return isInList(e, l->next);
}


/*--------------------*/


List
insertElem(Element e, List l)
{
    return isInList(e, l)
           ? l
           : cons(e, l);
}


/*--------------------*/

List
addElem(Element e, List l)
{
    return cons(e, l);
}

/*--------------------*/

static List
removeElem1(Element e, List l)
{
    if (isEmptyList(l))
        return l;

    if (eqElement(e, l->info))
    {
        return
            removeHead(l);
    }

    l->next = removeElem1(e, l->next);
    return l;
}

/*--------------------*/

/* a save version of removeElem */

List
removeElem(Element e, List l)
{
    List res = removeElem1(e, l);

    assert(invList(res));
    return res;
}
/*--------------------*/

static List
removeAllElems1(Element e, List l)
{
    if (isEmptyList(l))
        return l;

    if (eqElement(e, l->info))
    {
        return
            removeAllElems1(e, removeHead(l));
    }

    l->next = removeAllElems1(e, l->next);
    return l;
}

/*--------------------*/

/* a save version of removeAllElems */

List
removeAllElems(Element e, List l)
{
    List res = removeAllElems1(e, l);

    assert(invList(res));
    assert(! isInList(e, res));
    return res;
}

/*--------------------*/

void
freeList(List l)
{
    if (isEmptyList(l))
        return;

    freeList(l);
    removeHead(l);
}


/*--------------------*/
